;;;
;;; $Header: /home/gene/icfp2001/source/RCS/context.lisp,v 1.3 2001/07/29 09:17:32 gene Exp $
;;;

;;;
;;; Context is what the ICFP 2001 contest's documentation calls
;;; the "properties" of a decorated character.
;;;

(defstruct context
  (bold nil)
  (emphasis nil)
  (italic nil)
  (strong nil)
  (teletype nil)
  (underline 0)
  size
  color)

(defun child-context (tag old-context)
  (let ((child (if old-context
		   (copy-context old-context)
		 (make-context))))
    (cond ((equal tag "B")
	   (setf (context-bold child) t))
	  ((equal tag "EM")
	   (setf (context-emphasis child) (not (context-emphasis child))))
	  ((equal tag "I")
	   (setf (context-italic child) t))
	  ((equal tag "PL")
	   (setf (context-bold child) nil
		 (context-emphasis child) nil
		 (context-italic child) nil
		 (context-strong child) nil
		 (context-teletype child) nil
		 (context-underline child) 0))
	  ((equal tag "S")
	   (setf (context-strong child) t))
	  ((equal tag "TT")
	   (setf (context-teletype child) t))
	  ((equal tag "U")
	   (if (< (context-underline child) 3)
	       (incf (context-underline child))))
	  ((member tag '("0" "1" "2" "3" "4" "5" "6" "7" "8" "9")
		   :test #'equal)
	   (when *debug* (format t "~&child-context: debug: size tag ~A" tag))
	   (setf (context-size child) tag))
	  ((member tag '("r" "g" "b" "c" "m" "y" "k" "w") :test #'equal)
	   (when *debug*
	     (format t "~&child-context: debug: color tag ~A" tag))
	   (setf (context-color child) tag)))
    child))

(defun context-equal (c0 c1)
  (and (context-p c0)
       (context-p c1)
       (equal (context-bold c0) (context-bold c1))
       (equal (context-emphasis c0) (context-emphasis c1))
       (equal (context-italic  c0) (context-italic c1))
       (equal (context-strong  c0) (context-strong c1))
       (equal (context-teletype  c0) (context-teletype c1))
       (equal (context-underline  c0) (context-underline  c1))
       (equal (context-size c0) (context-size c1))
       (equal (context-color c0) (context-color c1))))

;;; --- end of file ---
